@@ -203,18 +203,25 @@ struct TimeToken : public Token
   virtual std::string getString(void*, ::ros::console::Level, const char*, const char*, const char*, int)
   {
     std::stringstream ss;
+    ss << ros::WallTime::now();
     if (ros::Time::isValid() && ros::Time::isSimTime())
     {
-      ss << ros::WallTime::now() << ", " << ros::Time::now();
-    }
-    else
-    {
-      ss << ros::WallTime::now();
+      ss << ", " << ros::Time::now();
     }
     return ss.str();
   }
 };
 
+struct WallTimeToken : public Token
+{
+  virtual std::string getString(void*, ::ros::console::Level, const char*, const char*, const char*, int)
+  {
+    std::stringstream ss;
+    ss << ros::WallTime::now();
+    return ss.str();
+  }
+};
+
 struct ThreadToken : public Token
 {
   virtual std::string getString(void*, ::ros::console::Level, const char*, const char*, const char*, int)
@@ -278,6 +285,10 @@ TokenPtr createTokenFromType(const std::string& type)
   {
     return TokenPtr(boost::make_shared<TimeToken>());
   }
+  else if (type == "walltime")
+  {
+    return TokenPtr(boost::make_shared<WallTimeToken>());
+  }
   else if (type == "thread")
   {
     return TokenPtr(boost::make_shared<ThreadToken>());
